'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NOT_ITERATOR_ERROR = undefined;
exports.default = proc;

var _utils = require('./utils');

var _io = require('./io');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var NOT_ITERATOR_ERROR = exports.NOT_ITERATOR_ERROR = "proc first argument must be an iterator";

function proc(iterator) {
  var subscribe = arguments.length <= 1 || arguments[1] === undefined ? function () {
    return function () {};
  } : arguments[1];
  var dispatch = arguments.length <= 2 || arguments[2] === undefined ? function () {} : arguments[2];

  (0, _utils.check)(iterator, _utils.is.iterator, NOT_ITERATOR_ERROR);

  var deferredInput = undefined,
      deferredEnd = undefined;
  var canThrow = _utils.is.throw(iterator);

  var endP = new Promise(function (resolve, reject) {
    return deferredEnd = { resolve: resolve, reject: reject };
  });

  var unsubscribe = subscribe(function (input) {
    if (deferredInput && deferredInput.match(input)) deferredInput.resolve(input);
  });

  next();
  iterator._isRunning = true;
  return endP;

  function next(arg, isError) {
    //console.log('next', arg, isError)
    deferredInput = null;
    try {
      if (isError && !canThrow) throw arg;
      var result = isError ? iterator.throw(arg) : iterator.next(arg);

      if (!result.done) {
        //console.log('yield', name, result.value)
        runEffect(result.value).then(next, function (err) {
          return next(err, true);
        });
      } else {
        //console.log('return', name, result.value)
        iterator._isRunning = false;
        iterator._result = result.value;
        unsubscribe();
        deferredEnd.resolve(result.value);
      }
    } catch (err) {
      //console.log('catch', name, err)
      iterator._isRunning = false;
      iterator._error = err;
      unsubscribe();
      deferredEnd.reject(err);
    }
  }

  function runEffect(effect) {
    var data = undefined;
    return _utils.is.array(effect) ? Promise.all(effect.map(runEffect)) : _utils.is.iterator(effect) ? proc(effect, subscribe, dispatch) : (data = _io.as.take(effect)) ? runTakeEffect(data) : (data = _io.as.put(effect)) ? runPutEffect(data) : (data = _io.as.race(effect)) ? runRaceEffect(data) : (data = _io.as.call(effect)) ? runCallEffect(data.fn, data.args) : (data = _io.as.cps(effect)) ? runCPSEffect(data.fn, data.args) : (data = _io.as.fork(effect)) ? runForkEffect(data.task, data.args) : (data = _io.as.join(effect)) ? runJoinEffect(data) : /* resolve anything else  */Promise.resolve(effect);
  }

  function runTakeEffect(pattern) {
    return new Promise(function (resolve) {
      deferredInput = { resolve: resolve, match: (0, _io.matcher)(pattern), pattern: pattern };
    });
  }

  function runPutEffect(action) {
    return Promise.resolve(1).then(function () {
      return dispatch(action);
    });
  }

  function runCallEffect(fn, args) {
    return !_utils.is.generator(fn) ? Promise.resolve(fn.apply(undefined, _toConsumableArray(args))) : proc(fn.apply(undefined, _toConsumableArray(args)), subscribe, dispatch);
  }

  function runCPSEffect(fn, args) {
    return new Promise(function (resolve, reject) {
      fn.apply(undefined, _toConsumableArray(args.concat(function (err, res) {
        return _utils.is.undef(err) ? resolve(res) : reject(err);
      })));
    });
  }

  function runForkEffect(task, args) {
    var _taskDesc;

    var _generator = undefined,
        _iterator = undefined;
    if (_utils.is.generator(task)) {
      _generator = task;
      _iterator = _generator.apply(undefined, _toConsumableArray(args));
    } else if (_utils.is.iterator(task)) {
      // directly forking an iterator
      _iterator = task;
    } else {
      //simple effect: wrap in a generator
      _iterator = regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return _utils.is.func(task) ? task.apply(undefined, _toConsumableArray(args)) : task;

              case 2:
                return _context.abrupt('return', _context.sent);

              case 3:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      })();
    }

    var _done = proc(_iterator, subscribe, dispatch);

    var taskDesc = (_taskDesc = {}, _defineProperty(_taskDesc, _utils.TASK, true), _defineProperty(_taskDesc, '_generator', _generator), _defineProperty(_taskDesc, '_iterator', _iterator), _defineProperty(_taskDesc, '_done', _done), _defineProperty(_taskDesc, 'name', _generator && _generator.name), _defineProperty(_taskDesc, 'isRunning', function isRunning() {
      return _iterator._isRunning;
    }), _defineProperty(_taskDesc, 'result', function result() {
      return _iterator._result;
    }), _defineProperty(_taskDesc, 'error', function error() {
      return _iterator._error;
    }), _taskDesc);
    return Promise.resolve(taskDesc);
  }

  function runJoinEffect(task) {
    return task._done;
  }

  function runRaceEffect(effects) {
    return Promise.race(Object.keys(effects).map(function (key) {
      return runEffect(effects[key]).then(function (result) {
        return _defineProperty({}, key, result);
      }, function (error) {
        return Promise.reject(_defineProperty({}, key, error));
      });
    }));
  }
}